/**
 * Company Name : 中贸促信息技术有限责任公司
 * Project Name:caccpitWeb
 * File Name:XssHttpServletRequestWrapper.java
 * Package Name:ccpit.base.utils
 * Date:2016年8月9日下午4:16:42
 * Copyright (c) 2016, dingwei@ccpit.org All Rights Reserved.
 *
*/

package org.ccpit.base.utils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

/**
 * ClassName:XssHttpServletRequestWrapper <br/>
 * Function: TODO Xss 过滤. <br/>
 * Reason:	 TODO ADD REASON. <br/>
 * Date:     2016年8月9日 下午4:16:42 <br/>
 * @author   dingwei
 * @version  
 * @since    JDK 1.6
 * @see 	 
 */
public class XssHttpServletRequestWrapper extends HttpServletRequestWrapper {
	
	HttpServletRequest orgRequest = null;

	public XssHttpServletRequestWrapper(HttpServletRequest request) {
		super(request);
		orgRequest = request;
	}

	/** 
     * 覆盖getParameter方法，将参数名和参数值都做xss过滤。<br/> 
     * 如果需要获得原始的值，则通过super.getParameterValues(name)来获取<br/> 
     * getParameterNames,getParameterValues和getParameterMap也可能需要覆盖 
     */ 
    @Override 
    public String getParameter(String name) {  
        String value = super.getParameter(xssEncode(name));  
        if (value != null) {  
            value = xssEncode(value);  
        }  
        return value;  
    }  
   
    /** 
     * 覆盖getHeader方法，将参数名和参数值都做xss过滤。<br/> 
     * 如果需要获得原始的值，则通过super.getHeaders(name)来获取<br/> 
     * getHeaderNames 也可能需要覆盖 
     */ 
    @Override 
    public String getHeader(String name) {  
   
        String value = super.getHeader(xssEncode(name));  
        if (value != null) {  
            value = xssEncode(value);  
        }  
        return value;  
    }  
   
    /** 
     * 将容易引起xss漏洞的半角字符直接替换成全角字符 
     *  
     * @param s 
     * @return 
     */ 
    private static String xssEncode(String s) {  

    	if (s == null || s.isEmpty()) {  
            return s;  
        }  
        StringBuilder sb = new StringBuilder(s.length() + 16);  
        for (int i = 0; i < s.length(); i++) {  
            char c = s.charAt(i);  
            switch (c) {  
            case '>':  
                sb.append("＞");// 转义大于号   a
                break;  
            case '<':  
                sb.append("＜");// 转义小于号   
                break;  
            case '\'':  
                sb.append("＇");// 转义单引号   
                break;  
            case '\"':  
                sb.append("＂");// 转义双引号   
                break;  
            case '&':  
                sb.append("＆");// 转义&   
                break;  
            default:  
                sb.append(c);  
                break;  
            }  
        }  
        return sb.toString();  
    }  
   
    /** 
     * 获取最原始的request 
     *  
     * @return 
     */ 
    public HttpServletRequest getOrgRequest() {  
        return orgRequest;  
    }  
   
    /** 
     * 获取最原始的request的静态方法 
     *  
     * @return 
     */ 
    public static HttpServletRequest getOrgRequest(HttpServletRequest req) {  
        if (req instanceof XssHttpServletRequestWrapper) {  
            return ((XssHttpServletRequestWrapper) req).getOrgRequest();  
        }  
   
        return req;  
    } 
}

